' Send any stored data in EEPROM through the serial port when turned
' on.  Gives you one minute to turn unit off before storing new
' data.  Stores up to 156 measurements, in urems/hr, with a one or
' four hour sampling rate for up to 26 days worth of measurements.
  
' Requires circuit with CD4040 and 74C157 ICs and
' Vellemen Geiger/Muller counter kit #K2645.

' This project is low power.  If you have an LCD display Panel,
' disconnect it to conserve battery power.  This project should
' run continuously for up to two months on a fresh 9V battery.

' The effective range of the measurements are 0 to 255 urems/hr.


Symbol  AB      =       6
Symbol  clr     =       7
Symbol  temp	=	b0
Symbol	hour	=	b1
Symbol  mint	=	b2	
Symbol  mast	=	w4
Symbol	nsub	=	w6

begin:	for temp = 0 to 156
	read temp,b12
	serout AB,N2400,(#b12,10,13)		'send data numerically
	next temp				'followed by a cr and lf
loop:	sleep 60
	let dirs = %11100000			'Set up I/O directions. 
	low ab
	let nsub = pins & %00001111		'Take low and high nibble
	high ab					'readings from the 74C157
	let nsub = pins & %00011111 * 16 + nsub
	pulsout clr,2				'Clear CD4040
	let mast = mast + nsub * 100 / 15	'Accumulate urems/hr
	let mint = mint + 1			'Count the minutes
	if mint < 60 then loop			'60 = 1 hour, use 240 to
	let b11 = mast / mint			'log every 4 hours instead.
	write hour,b11				'Store the readings
	let hour = hour + 1
	let mint = 0				'Clear for next time
	let mast = 0
	if hour < 157 then loop
	end					'Low power forever.